@@ -1,5 +1,6 @@
 import requests
 from time import time, sleep
+import sys
 
 import pytest
 
@@ -29,3 +30,22 @@ def test_BokehWebInterface(loop):
                 sleep(0.01)
     with pytest.raises(Exception):
         response = requests.get('http://127.0.0.1:8787/status/')
+
+
+def test_bokeh_shutsdown_without_cluster___del__(loop):
+    c = LocalCluster(2, loop=loop, scheduler_port=0,
+                     services={('http', 0): HTTPScheduler},
+                     diagnostic_port=None)
+    proc = c.diagnostics.process
+    # don't run the del, as it isn't ever run in python < 3.5 due to cycles
+    c.__del__ = lambda self: None
+    del c
+    start = time()
+    while True:
+        if sys.version_info[0] >= 3:
+            if not proc.is_alive():
+                break
+        elif proc.poll() is not None:
+            break
+        assert time() < start + 5
+        sleep(0.01)
